package ru.dbotthepony.kstarbound.lua;

import com.kenai.jffi.Closure;
import jnr.ffi.LibraryLoader;
import jnr.ffi.Pointer;
import jnr.ffi.Runtime;
import jnr.ffi.annotations.LongLong;
import org.jetbrains.annotations.NotNull;

import javax.annotation.Nullable;

@SuppressWarnings({"UnnecessaryModifier", "SpellCheckingInspection", "unused"})
public interface LuaJNR {
	public int lua_pcallk(@NotNull Pointer luaState, int numArgs, int numResults, int msgh, @LongLong long ctx, @LongLong long callback);
	public int lua_callk(@NotNull Pointer luaState, int numArgs, int numResults, @LongLong long ctx, @LongLong long callback);
	public long lua_atpanic(@NotNull Pointer luaState, @LongLong long fn);

	@Nullable
	public Pointer luaL_newstate();
	public void lua_close(@NotNull Pointer luaState);

	// Стандартные библиотеки
	public void luaopen_base(@NotNull Pointer luaState);
	public void luaopen_package(@NotNull Pointer luaState);
	public void luaopen_coroutine(@NotNull Pointer luaState);
	public void luaopen_table(@NotNull Pointer luaState);
	public void luaopen_io(@NotNull Pointer luaState);
	public void luaopen_os(@NotNull Pointer luaState);
	public void luaopen_string(@NotNull Pointer luaState);
	public void luaopen_math(@NotNull Pointer luaState);
	public void luaopen_utf8(@NotNull Pointer luaState);
	public void luaopen_debug(@NotNull Pointer luaState);

	@Nullable
	public Pointer lua_tolstring(@NotNull Pointer luaState, int index, @LongLong long size);

	public int lua_load(@NotNull Pointer luaState, @LongLong long reader, long userData, @NotNull String chunkName, @NotNull String mode);

	public interface lua_CFunction extends Closure {
		int invoke(@NotNull Pointer luaState);

		@Override
		default void invoke(Buffer buffer) {
			//buffer.setIntReturn(invoke());
		}
	}

	public interface lua_Reader extends Closure {
		public long readNextChunk(@NotNull Pointer luaState, long userData, @NotNull Pointer sizeToRead);

		@Override
		default void invoke(Buffer buffer) {
			throw new RuntimeException();
		}
	}

	public interface lua_KFunction extends Closure {
		public int invoke(@NotNull Pointer luaState, int status, long context);

		@Override
		default void invoke(Buffer buffer) {
			throw new RuntimeException();
		}
	}

	// Операции над стаком
	// загрузка значений из Java на стек
	public void lua_createtable(@NotNull Pointer luaState, int arraySize, int hashSize);
	public void lua_pushnil(@NotNull Pointer luaState);
	public void lua_pushnumber(@NotNull Pointer luaState, double value);
	public void lua_pushinteger(@NotNull Pointer luaState, @LongLong long value);
	public void lua_pushboolean(@NotNull Pointer luaState, int value);
	public void lua_pushglobaltable(@NotNull Pointer luaState);

	// NUL терминированная строка
	public void lua_pushstring(@NotNull Pointer luaState, @NotNull String value);

	// двоичная строка
	public long lua_pushlstring(@NotNull Pointer luaState, @LongLong long stringPointer, @LongLong long length);

	// Загрузка Lua значений на стек
	public int lua_getglobal(@NotNull Pointer luaState, @NotNull String name);

	// запись значений со стека
	public void lua_settable(@NotNull Pointer luaState, int stackIndex);
	public void lua_setglobal(@NotNull Pointer luaState, @NotNull String name);

	// проверка стека
	public int lua_checkstack(@NotNull Pointer luaState, int value);
	public int lua_absindex(@NotNull Pointer luaState, int value);
	public int lua_iscfunction(@NotNull Pointer luaState, int index);
	public int lua_isinteger(@NotNull Pointer luaState, int index);
	public int lua_isnumber(@NotNull Pointer luaState, int index);
	public int lua_isstring(@NotNull Pointer luaState, int index);
	public int lua_isuserdata(@NotNull Pointer luaState, int index);

	public int lua_toboolean(@NotNull Pointer luaState, int index);
	public int lua_tocfunction(@NotNull Pointer luaState, int index);
	public int lua_toclose(@NotNull Pointer luaState, int index);
	public int lua_gettable(@NotNull Pointer luaState, int index);

	@LongLong
	public long lua_tointegerx(@NotNull Pointer luaState, int index, @LongLong long successCode);
	public double lua_tonumberx(@NotNull Pointer luaState, int index, @LongLong long successCode);

	public int lua_settop(@NotNull Pointer luaState, int index);

	public int lua_next(@NotNull Pointer luaState, int index);
	public int lua_type(@NotNull Pointer luaState, int index);

	/**
	 * Returns the index of the top element in the stack.
	 * Because indices start at 1, this result is equal to the number of elements in the stack; in particular, 0 means an empty stack.
	 */
	public int lua_gettop(@NotNull Pointer luaState);

	@NotNull
	public static final LuaJNR INSTANCE = LibraryLoader.create(LuaJNR.class).load("lua54");

	@NotNull
	public static final Runtime RUNTIME = Runtime.getRuntime(INSTANCE);
}
