package ru.dbotthepony.kstarbound

import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kstarbound.defs.MovementParameters
import ru.dbotthepony.kstarbound.defs.player.PlayerMovementParameters
import ru.dbotthepony.kstarbound.util.AssetPathStack
import java.util.concurrent.ForkJoinPool
import java.util.concurrent.ForkJoinTask
import kotlin.reflect.KMutableProperty0

object GlobalDefaults {
	private val LOGGER = LogManager.getLogger()

	var playerMovementParameters = PlayerMovementParameters()
		private set

	var movementParameters = MovementParameters()
		private set

	private object EmptyTask : ForkJoinTask<Unit>() {
		private fun readResolve(): Any = EmptyTask
		override fun getRawResult() {
		}

		override fun setRawResult(value: Unit?) {
		}

		override fun exec(): Boolean {
			return true
		}
	}

	private inline fun <reified T> load(path: String, accept: KMutableProperty0<T>, executor: ForkJoinPool): ForkJoinTask<*> {
		val file = Starbound.locate(path)

		if (!file.exists) {
			LOGGER.fatal("$path does not exist, expect bad things to happen!")
			return EmptyTask
		} else if (!file.isFile) {
			LOGGER.fatal("$path is not a file, expect bad things to happen!")
			return EmptyTask
		} else {
			return executor.submit {
				AssetPathStack("/") {
					accept.set(Starbound.gson.fromJson(file.jsonReader(), T::class.java))
				}
			}
		}
	}

	fun load(log: ILoadingLog, executor: ForkJoinPool): List<ForkJoinTask<*>> {
		val tasks = ArrayList<ForkJoinTask<*>>()

		tasks.add(load("/default_actor_movement.config", ::playerMovementParameters, executor))
		tasks.add(load("/default_movement.config", ::movementParameters, executor))

		return listOf(executor.submit {
			val line = log.line("Loading global defaults...")
			val time = System.nanoTime()
			tasks.forEach { it.join() }
			line.text = "Loaded global defaults in ${((System.nanoTime() - time) / 1_000_000.0).toLong()}ms"
		})
	}
}
