package ru.dbotthepony.kstarbound

import it.unimi.dsi.fastutil.ints.IntIterators
import it.unimi.dsi.fastutil.objects.ObjectIterators

interface ILoadingLog : Iterable<String> {
	interface ILine {
		var text: String
	}

	fun line(text: String): ILine

	companion object : ILoadingLog, ILine {
		override var text: String
			get() = ""
			set(value) {}

		override fun line(text: String): ILine {
			return this
		}

		override fun iterator(): Iterator<String> {
			return ObjectIterators.emptyIterator()
		}
	}
}

class LoadingLog : ILoadingLog {
	private val lines = arrayOfNulls<Line>(128)
	private var index = 0
	private val lock = Any()
	private var size = 0
	var lastActivity: Long = System.nanoTime()
		private set

	override fun line(text: String): ILoadingLog.ILine {
		return Line(text)
	}

	inner class Line(text: String) : ILoadingLog.ILine {
		override var text: String = text
			set(value) {
				field = value
				lastActivity = System.nanoTime()
			}

		init {
			lastActivity = System.nanoTime()

			synchronized(lock) {
				lines[index++ and 127] = this
				size = (size + 1).coerceAtMost(127)
			}
		}
	}

	override fun iterator(): Iterator<String> {
		return object : Iterator<String> {
			private val index = (this@LoadingLog.index - 1) and 127
			private val parent = IntIterators.fromTo(0, size)

			override fun hasNext(): Boolean {
				return parent.hasNext()
			}

			override fun next(): String {
				return lines[(index - parent.nextInt()) and 127]!!.text
			}
		}
	}
}
