package ru.dbotthepony.kstarbound.client.gl.vertex

import org.lwjgl.opengl.GL45
import ru.dbotthepony.kstarbound.client.StarboundClient
import ru.dbotthepony.kstarbound.client.gl.VertexArrayObject
import ru.dbotthepony.kstarbound.client.gl.BufferObject
import ru.dbotthepony.kstarbound.client.gl.checkForGLError

/**
 * Быстрое наполнение буфера вершинами, загрузка в память видеокарты, и отрисовка
 */
class StreamVertexBuilder(
	attributes: VertexAttributes,
	type: GeometryType? = null,
	initialCapacity: Int = 32,
) {
	val state = StarboundClient.current()
	val builder = VertexBuilder(attributes, type, initialCapacity)
	private val vao = VertexArrayObject()
	private val vbo = BufferObject.VBO()
	private val ebo = BufferObject.EBO()

	init {
		vao.elementBuffer = ebo
		vao.bindAttributes(vbo, attributes)
	}

	fun upload(drawType: Int = GL45.GL_DYNAMIC_DRAW) {
		builder.upload(vbo, ebo, drawType)
	}

	fun bind() = vao.bind()
	fun unbind() = vao.unbind()

	fun draw(primitives: Int = GL45.GL_TRIANGLES) {
		bind()
		GL45.glDrawElements(primitives, builder.indexCount, builder.indexType, 0L)
		checkForGLError()
		unbind()
	}
}
