package ru.dbotthepony.kstarbound.defs

import com.google.common.collect.ImmutableSet
import com.google.gson.Gson
import com.google.gson.JsonObject
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.io.json.consumeNull
import ru.dbotthepony.kstarbound.util.contains
import ru.dbotthepony.kstarbound.util.get

enum class StatModifierType(vararg names: String) {
	BASE_ADDITION("value", "baseAddition"),
	BASE_MULTIPLICATION("baseMultiplier"),

	OVERALL_ADDITION("effectiveValue", "effectiveAddition"),
	OVERALL_MULTIPLICATION("effectiveMultiplier");

	val names: ImmutableSet<String> = ImmutableSet.copyOf(names)
}

data class StatModifier(val stat: String, val value: Double, val type: StatModifierType) {
	class Adapter(gson: Gson) : TypeAdapter<StatModifier>() {
		private val objects = gson.getAdapter(JsonObject::class.java)

		override fun write(out: JsonWriter, value: StatModifier?) {
			if (value == null) {
				out.nullValue()
			} else {
				out.beginObject()
				out.name("stat")
				out.value(value.stat)
				out.name(value.type.names.first())
				out.value(value.value)
				out.endObject()
			}
		}

		override fun read(`in`: JsonReader): StatModifier? {
			if (`in`.consumeNull()) {
				return null
			} else {
				val read = objects.read(`in`)

				val stat = read["stat"]?.asString ?: throw JsonSyntaxException("No stat to modify specified")

				for (type in StatModifierType.entries)
					for (name in type.names)
						if (name in read)
							return StatModifier(stat, read.get(name, 0.0), type)

				throw JsonSyntaxException("Not a stat modifier: $read")
			}
		}
	}
}
