package ru.dbotthepony.kstarbound.defs.player

import com.google.common.collect.ImmutableList
import com.google.common.collect.ImmutableSet
import ru.dbotthepony.kstarbound.defs.BaseMovementParameters
import ru.dbotthepony.kstarbound.defs.JumpProfile
import ru.dbotthepony.kstarbound.io.json.builder.JsonAlias
import ru.dbotthepony.kstarbound.io.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.io.json.builder.JsonFlat
import ru.dbotthepony.kstarbound.util.KOptional
import ru.dbotthepony.kvector.vector.Vector2d

@JsonFactory
data class PlayerMovementParameters(
	@JsonFlat
	val base: BaseMovementParameters.Impl = BaseMovementParameters.Impl(),

	@JsonAlias("collisionPoly")
	val standingPoly: KOptional<ImmutableList<Vector2d>> = KOptional.empty(),
	@JsonAlias("collisionPoly")
	val crouchingPoly: KOptional<ImmutableList<Vector2d>> = KOptional.empty(),

	val walkSpeed: KOptional<Double> = KOptional.empty(),
	val runSpeed: KOptional<Double> = KOptional.empty(),
	val flySpeed: KOptional<Double> = KOptional.empty(),

	val minimumLiquidPercentage: KOptional<Double> = KOptional.empty(),
	val liquidImpedance: KOptional<Double> = KOptional.empty(),
	val normalGroundFriction: KOptional<Double> = KOptional.empty(),
	val ambulatingGroundFriction: KOptional<Double> = KOptional.empty(),
	val groundForce: KOptional<Double> = KOptional.empty(),
	val airForce: KOptional<Double> = KOptional.empty(),
	val liquidForce: KOptional<Double> = KOptional.empty(),

	val airJumpProfile: JumpProfile = JumpProfile(),
	val liquidJumpProfile: JumpProfile = JumpProfile(),

	val fallStatusSpeedMin: KOptional<Double> = KOptional.empty(),
	val fallThroughSustainFrames: KOptional<Int> = KOptional.empty(),

	val groundMovementMinimumSustain: KOptional<Double> = KOptional.empty(),
	val groundMovementMaximumSustain: KOptional<Double> = KOptional.empty(),
	val groundMovementCheckDistance: KOptional<Double> = KOptional.empty(),

	val pathExploreRate: KOptional<Double> = KOptional.empty(),
) : BaseMovementParameters by base {
	fun merge(other: PlayerMovementParameters): PlayerMovementParameters {
		return PlayerMovementParameters(
			base = base.merge(other.base),
			standingPoly = standingPoly.or(other.standingPoly),
			crouchingPoly = crouchingPoly.or(other.crouchingPoly),
			walkSpeed = walkSpeed.or(other.walkSpeed),
			runSpeed = runSpeed.or(other.runSpeed),
			flySpeed = flySpeed.or(other.flySpeed),
			minimumLiquidPercentage = minimumLiquidPercentage.or(other.minimumLiquidPercentage),
			liquidImpedance = liquidImpedance.or(other.liquidImpedance),
			normalGroundFriction = normalGroundFriction.or(other.normalGroundFriction),
			ambulatingGroundFriction = ambulatingGroundFriction.or(other.ambulatingGroundFriction),
			groundForce = groundForce.or(other.groundForce),
			airForce = airForce.or(other.airForce),
			liquidForce = liquidForce.or(other.liquidForce),
			airJumpProfile = airJumpProfile.merge(other.airJumpProfile),
			liquidJumpProfile = liquidJumpProfile.merge(other.liquidJumpProfile),
			fallStatusSpeedMin = fallStatusSpeedMin.or(other.fallStatusSpeedMin),
			fallThroughSustainFrames = fallThroughSustainFrames.or(other.fallThroughSustainFrames),
			groundMovementMinimumSustain = groundMovementMinimumSustain.or(other.groundMovementMinimumSustain),
			groundMovementMaximumSustain = groundMovementMaximumSustain.or(other.groundMovementMaximumSustain),
			groundMovementCheckDistance = groundMovementCheckDistance.or(other.groundMovementCheckDistance),
		)
	}
}
