package ru.dbotthepony.kstarbound.io

import org.lwjgl.stb.STBIEOFCallback
import org.lwjgl.stb.STBIIOCallbacks
import org.lwjgl.stb.STBIReadCallback
import org.lwjgl.stb.STBIReadCallbackI
import org.lwjgl.stb.STBISkipCallback
import org.lwjgl.system.MemoryUtil
import java.io.InputStream

fun stream2STBIO(stream: InputStream): STBIIOCallbacks {
	require(stream.markSupported()) { "Provided stream must support mark" }

	val callback = STBIIOCallbacks.create()

	val read = STBIReadCallback.create(STBIReadCallbackI { _, buf, size ->
		val readBuf = ByteArray(size)
		val read = stream.read(readBuf)

		for (i in 0 until read) {
			MemoryUtil.memPutByte(buf + i, readBuf[i])
		}

		return@STBIReadCallbackI read
	})

	val skip = STBISkipCallback.create { _, n -> stream.skip(n.toLong()) }

	val eof = STBIEOFCallback.create {
		stream.mark(1)
		val empty = stream.read() == -1
		stream.reset()
		if (empty) 1 else 0
	}

	callback.set(read, skip, eof)
	return callback
}
