package ru.dbotthepony.kstarbound.io.json

import com.google.gson.Gson
import com.google.gson.JsonElement
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.internal.bind.JsonTreeReader
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.util.Either
import java.lang.reflect.ParameterizedType

object EitherTypeAdapter : TypeAdapterFactory {
	override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
		if (type.rawType == Either::class.java) {
			val params = type.type as? ParameterizedType ?: return null
			val (left, right) = params.actualTypeArguments

			return object : TypeAdapter<Either<Any, Any>>() {
				private val leftAdapter = gson.getAdapter(TypeToken.get(left)) as TypeAdapter<Any?>
				private val rightAdapter = gson.getAdapter(TypeToken.get(right)) as TypeAdapter<Any?>
				private val elemAdapter = gson.getAdapter(JsonElement::class.java)

				override fun write(out: JsonWriter, value: Either<Any, Any>?) {
					if (value == null)
						out.nullValue()
					else
						value.map({ leftAdapter.write(out, it) }, { rightAdapter.write(out, it) })
				}

				override fun read(`in`: JsonReader): Either<Any, Any>? {
					if (`in`.peek() == JsonToken.NULL)
						return null

					val elem = elemAdapter.read(`in`)

					return try {
						Either.left(leftAdapter.read(JsonTreeReader(elem)) ?: throw NullPointerException("left was empty"))
					} catch(leftError: Throwable) {
						try {
							Either.right(rightAdapter.read(JsonTreeReader(elem)) ?: throw NullPointerException("right was empty"))
						} catch(rightError: Throwable) {
							val error = JsonSyntaxException("Can't read Either of values (left is $left, right is $right)")
							error.addSuppressed(leftError)
							error.addSuppressed(rightError)
							throw error
						}
					}
				}
			} as TypeAdapter<T>
		}

		return null
	}
}
