package ru.dbotthepony.kstarbound.io.json

import com.google.gson.Gson
import com.google.gson.JsonElement
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.util.OneOf
import java.lang.reflect.ParameterizedType

object OneOfTypeAdapter : TypeAdapterFactory {
	private class Adapter<A, B, C>(gson: Gson, a: TypeToken<A>, b: TypeToken<B>, c: TypeToken<C>) : TypeAdapter<OneOf<A, B, C>>() {
		private val adapter0 = gson.getAdapter(a)
		private val adapter1 = gson.getAdapter(b)
		private val adapter2 = gson.getAdapter(c)
		private val elements = gson.getAdapter(JsonElement::class.java)

		override fun write(out: JsonWriter, value: OneOf<A, B, C>?) {
			if (value == null) {
				out.nullValue()
			} else {
				value.v0.ifPresent { adapter0.write(out, it) }
				value.v1.ifPresent { adapter1.write(out, it) }
				value.v2.ifPresent { adapter2.write(out, it) }
			}
		}

		override fun read(`in`: JsonReader): OneOf<A, B, C>? {
			if (`in`.consumeNull()) {
				return null
			} else {
				val read = elements.read(`in`)
				val errors = ArrayList<Throwable>(3)

				try {
					return OneOf.first(adapter0.fromJsonTree(read))
				} catch (err: Exception) {
					errors.add(err)
				}

				try {
					return OneOf.second(adapter1.fromJsonTree(read))
				} catch (err: Exception) {
					errors.add(err)
				}

				try {
					return OneOf.third(adapter2.fromJsonTree(read))
				} catch (err: Exception) {
					errors.add(err)
				}

				throw JsonSyntaxException("None of type adapters consumed the input: $read").also { errors.forEach(it::addSuppressed) }
			}
		}
	}

	override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
		if (type.rawType === OneOf::class.java) {
			val p = type.type as? ParameterizedType ?: return null
			val (a, b, c) = p.actualTypeArguments

			return Adapter(gson, TypeToken.get(a), TypeToken.get(b), TypeToken.get(c)) as TypeAdapter<T>
		}

		return null
	}
}