package ru.dbotthepony.kstarbound.lua

import com.google.gson.JsonElement
import com.google.gson.JsonNull
import it.unimi.dsi.fastutil.objects.ObjectOpenHashSet
import ru.dbotthepony.kstarbound.util.traverseJsonPath

fun loadInternalScript(name: String): String {
	return LuaState::class.java.getResourceAsStream("/scripts/$name.lua")?.readAllBytes()?.toString(Charsets.UTF_8) ?: throw RuntimeException("/scripts/$name.lua is missing!")
}

private val messageHandlerLua by lazy { loadInternalScript("message_handler") }
private val configLua by lazy { loadInternalScript("config") }

fun LuaState.exposeConfig(config: JsonElement) {
	load(configLua, "@starbound.jar!/scripts/config.lua")
	call()

	loadGlobal("config")
	val table = stackTop
	push("_get")
	push {
		push(traverseJsonPath(getString(), config))
		return@push 1
	}

	setTableValue(table)
	pop()
}

class MessageHandler(val lua: LuaState) {
	init {
		lua.load(messageHandlerLua, "@starbound.jar!/scripts/message_handler.lua")
		lua.call()

		lua.loadGlobal("message")

		lua.setTableClosure("subscribe", this) { subscriptions.add(it.getString()) }

		lua.pop()
	}

	private val subscriptions = ObjectOpenHashSet<String>()

	fun isSubscribed(name: String): Boolean {
		return name in subscriptions
	}

	fun call(name: String, vararg values: JsonElement): JsonElement? {
		if (!isSubscribed(name))
			return null

		lua.loadGlobal("message")
		lua.loadTableValue("call")

		lua.push(name)

		for (value in values)
			lua.push(value)

		lua.call(numArgs = 1 + values.size, numResults = 1)
		val value = lua.getValue()
		lua.pop() // message
		return value
	}
}
