package ru.dbotthepony.kstarbound.math

import kotlin.math.absoluteValue

fun lerp(t: Double, a: Double, b: Double): Double {
	return a * (1.0 - t) + b * t
}

/**
 * Выполняет преобразование [value] типа [Double] в [Int] так,
 * что выходной [Int] всегда будет больше или равен по модулю [value]
 */
fun roundByAbsoluteValue(value: Double): Int {
	if (value > 0.0) {
		if (value % 1.0 != 0.0) {
			return value.toInt() + 1
		}

		return value.toInt()
	} else if (value == 0.0) {
		return 0
	} else {
		if (value % 1.0 != -0.0) {
			return value.toInt() - 1
		}

		return value.toInt()
	}
}

fun roundTowardsNegativeInfinity(value: Double): Int {
	if (value == 0.0)
		return 0

	if (value > 0.0) {
		return value.toInt()
	}

	if (value % 1.0 != -0.0) {
		return value.toInt() - 1
	}

	return value.toInt()
}

fun roundTowardsPositiveInfinity(value: Double): Int {
	if (value == 0.0)
		return 0

	if (value < 0.0) {
		return value.toInt()
	}

	if (value % 1.0 != 0.0) {
		return value.toInt() + 1
	}

	return value.toInt()
}

fun divideUp(a: Int, b: Int): Int {
	return if (a % b == 0) {
		a / b
	} else {
		a / b + 1
	}
}

private const val EPSILON = 0.00000001

fun weakCompare(a: Double, b: Double, epsilon: Double = EPSILON): Int {
	if (a == b || (a - b).absoluteValue < epsilon) {
		return 0
	}

	if (a > b)
		return 1

	return -1
}

fun weakDoubleZeroing(value: Double, epsilon: Double = EPSILON): Double {
	if (weakCompare(value, 0.0, epsilon) == 0)
		return 0.0

	return value
}
