package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kvector.api.IStruct2i

// for getting tiles directly, avoiding manual layer specification
interface ITileAccess : ICellAccess {
	// relative
	fun getTile(x: Int, y: Int): ITileState?
	fun getTile(pos: IStruct2i) = getTile(pos.component1(), pos.component2())
	fun getTileDirect(x: Int, y: Int): ITileState?
	fun getTileDirect(pos: IStruct2i) = getTile(pos.component1(), pos.component2())
}

sealed class TileView(parent: ICellAccess) : ITileAccess, ICellAccess by parent {
	class Foreground(parent: ICellAccess) : TileView(parent) {
		override fun getTile(x: Int, y: Int): ITileState? {
			return getCell(x, y)?.foreground
		}

		override fun getTileDirect(x: Int, y: Int): ITileState? {
			return getCellDirect(x, y)?.foreground
		}
	}

	class Background(parent: ICellAccess) : TileView(parent) {
		override fun getTile(x: Int, y: Int): ITileState? {
			return getCell(x, y)?.background
		}

		override fun getTileDirect(x: Int, y: Int): ITileState? {
			return getCellDirect(x, y)?.background
		}
	}
}
