package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kstarbound.Registries
import ru.dbotthepony.kstarbound.RegistryObject
import ru.dbotthepony.kstarbound.defs.tile.BuiltinMetaMaterials
import ru.dbotthepony.kstarbound.defs.tile.MaterialModifier
import ru.dbotthepony.kstarbound.defs.tile.TileDefinition
import java.io.DataInputStream

interface ITileState {
	var material: TileDefinition
	var modifier: MaterialModifier?
	var color: TileColor
	var hueShift: Float
	var modifierHueShift: Float

	fun setHueShift(value: Int) {
		if (value < 0) {
			hueShift = 0f
		} else if (value >= 255) {
			hueShift = 360f
		} else {
			hueShift = value / 255f * 360f
		}
	}

	fun setModHueShift(value: Int) {
		if (value < 0) {
			modifierHueShift = 0f
		} else if (value >= 255) {
			modifierHueShift = 360f
		} else {
			modifierHueShift = value / 255f * 360f
		}
	}

	fun read(stream: DataInputStream) {
		material = Registries.tiles[stream.readUnsignedShort()]?.value ?: BuiltinMetaMaterials.EMPTY
		setHueShift(stream.read())
		color = TileColor.of(stream.read())
		modifier = Registries.tileModifiers[stream.readUnsignedShort()]?.value
		setModHueShift(stream.read())
	}

	companion object {
		fun skip(stream: DataInputStream) {
			stream.skipNBytes(2 + 1 + 1 + 2 + 1)
		}
	}
}